c
      program pframe
c
c ********************************************************************
c
c    A    P R O G R A M
c
c         to SOLVE PLANE FRAMES in FINITE DISPLACEMENTS by FEM
c
c     finite axial extension effect(exact equilibrium) can be included 
c
c                              2008/06/27 Axial Force and Shear Output
c                                         Calculation and Mistake in Sign
c                 T. Iwakuma   09/03/91 (all models combined in one)
c                              02/07/90, 10/05/89 ** for NEWS **
c                              08/19/88 ** Finite Extension Effect **
c                              07/01/86, 06/26/86
c                              * Error was found in Determinant calc.
c                              06/19/86(Position Neg.Elm.Tang.Stiff.)
c                              04/28/86(Output control change)
c                              02/10/86, 12/14/85, 11/12/85, 11/10/85
c                                        ( FRM_ARCS.FOR : 11/07/85)
c
c ********************************************************************
c
c                  - - - -   REMARKS   - - - -
c
c      Local stiffness matrix is the one for Timoshenko beam
c        including an approximate geometric stiffness matrix in
c            "A linearized Timoshenko beam theory in finite
c             displacements," by A.H.,T.I. and S.K., Proc. JSCE,
c             Struc.Eng./Earthq.Eng.,Vol.2,No.2,pp.321s-326s,1985.
c        with extension effect in
c            "Timoshenko beam theory with extension effect and
c             its stiffness equation for finite rotation," by T.I.
c             Computers & Structures, Vol.34, pp.239-250, 1990.
c         Model A = 'SPRING' model leads to modified buckling formula
c         Model B = 'BAR' model leads to Engesser's buckling formula
c
c      Any part of a frame can be to any direction,
c        and a couple of hinges can be inserted in the structure.
c
c      Skyline method is used to store non-symmetric matrix.
c        Elimination method is used to solve the simultaneous
c           algebraic equation.
c            "Skyline algorithms for multilevel substructure
c             analysis," by A.E.E. and D.W.M., Int.J.Numer.Meth.Eng.,
c             Vol.21,pp.465-479, 1985.
c
c      Arc length method is used to overcome
c        difficulties near some singular points.
c            "Analysis of elastic buckling problem by arc length
c             method; Part 1 and 2," by T.H., Trans.Archit.Inst.Japan,
c             No.242 (pp.41-50) and 243 (pp.21-31), 1976.
c
c      Change of arc length and modification of force vector
c                              are used to avoid too many iterations.
c
c      In case the program is interrupted by some accident
c        such as power failure, one can continue the computation
c          as a continued case by using 'pframe.cnt' and by appropriate
c                                                 change of data (2).
c
c ********************************************************************
c
c                    - - - -   DATA   - - - -
c
c  A file of the following set of data must be created as 'pframe.dat'.
c
c   (A) Basic data section
c      (1)  20A4       title(20) = title of the case
c                       this also controls the execution; see (E)
c      (2)  3I5        n     = number of elements <= 140
c                      node  = number of nodes <= 128
c                      icont = indicates a continued computation
c                            = 0   if this is a new case
c                            <>0   if this is a continued case
c                                   in this case, 'pframe.cnt' must exist.
c      (3)  3I5        iel        = element # from 1 to n
c                      nnlft(iel) = node # left of this element
c                      nnrgt(iel) = node # right of this element
c           Need "n" sets of this data (3)
c      (4)  3I5        ioptg = optional geometric stiffness
c                            = 0 if not included, <>0 if included
c                      iext  = optional extension effect
c                            = 0 if not included, <>0 if included
c                              when iext<>0, ioptg is set to 1 (obvious)
c                      model =  0 if 'BAR' model [B: V=(GkA+N)gamma]
c                            <> 0 if 'SPRING' model [A: V=GkA gamma]
c      (5)  I5         iang = no. of different members <= 64
c      (6)  I5,5F10.0  ielang(i) = same member up to this element
c                      ang(i)    = angle of those elements (deg)
c                       x-axis lies horizontally, y- vertically from
c                        up to down; angle is measured clockwise
c                         from x-axis toward y-axis
c                      ratio(i)  = basic length / element length
c                      bet(i)    = slenderness ratio
c                                    in terms of basic length
c                      alph(i)   = Young-s modulus/shear modulus
c                       ( alph=0. for the Bernoulli-Euler beam )
c                      ratiob(i) = EI / (basic EI)
c                                = bending rigidity ratio
c                       all vectors are non-dimensionalized by
c                        the basic length and the basic EI.
c           Need "iang" sets of this data (6)
c      (7)  I5         nhng = number of left-hinged elements <= 80
c                           = 0 if no hinge exists
c      (8)  16I5       ielhng(i) = left-hinged element no.
c                                         in its ascending order
c           Need "nhng" sets of this data (8)
c      (9)  3I5,F10.0  iep  = negative power of tolerance for iteration
c                      nout = every "nout"-th element will be output
c                      itmx = max. no. of iteraton for one trial
c                           = 10 if not specified in data file
c                      r    = arc length
c                              must be a negative value when automatic
c                              modification of 'ddf' should be omitted.
c      (10) I5         ninc = total no. of incremental steps
c   (B) Incremental load data (B.C.) section
c      (11) I5         nlinc = no. of loads >= 0, and <= 64
c      (12) 2I5,F10.0  i = node no. where incremental load is applied
c                      idf(i) = 1 for horizontal force,
c                               2 for vertical, 3 for moment
c                      ddf(i) = component (ratio) of load vector
c                                 actual load = p * ddf(i)
c           Need "nlinc" sets of this data (12)
c   (C) Constant displacement data (B.C.) section
c      (13) I5         ndcon = no. of disp. >= 0, and <= 64
c      (14) 2I5,F10.0  similar to (12)
c           Need "ndcon" sets of this data (14)
c   (D) Graphics output control section
c      (15) 2I5        nlgrph = load # to be output for graphics data
c                                      <= nlinc of data (11)
c                      ndgrph = # of internal-force or displ. data
c                               to be output <=8
c      (16) 16I5       negrph = element # of output <= n
c                      nfgrph = # to specify components at the
c                               left or right node of element #negrph
c                                 1       4   : horizontal displ.
c                                 2       5   : vertical   displ.
c                                 3       6   : rotational displ.
c                                 7      10   : axial force
c                                 8      11   : shear force
c                                 9      12   : bending moment
c           Need "ndgrph" pairs of these data in one line
c
c   Data from (A) through (D) is a set for one case. Give as many as
c      one likes. But at the end of all sets of data,
c         one must put a line of data as follows:
c   (E) Control data to stop execution at the end of entire data
c      (17) A4         title(1) = 'end ' to stop execution
c
c   Set(s) of data from (A) through (D) and a line of data (E) are
c                     necessary for one execution without any errors.
c
c ********************************************************************
c
c                    - - - -   FILES   - - - -
c
c           PFRAME.DAT : Data File (explained above)
c           PFRAME.RES : Results (to printer)
c           PFRAME.GRP : Data for Graphics (for N-Graph etc.)
c           PFRAME.CNT : Data to continue the case (unformatted)
c           PFRAME.MON : Monitoring (CON for PC)
c
c ********************************************************************
c
c                   - - - -   PROGRAM   - - - -
c
      implicit real*8 (a-h,o-z)
      character*4 aend,title(20)
      dimension a(1153),ia(1436)
      common / workb / b(8000)
      common / workc / c(8000)
      data aend / 'end ' /
c
c max. size of 'a', 'b' and 'ia'
c   na>=9*node+1 , nb=max. size possible (64KB for PC),
c   la>=2*n+9*node+4
c
c  limits on data above can be relaxed by changing na, nb and la,
c                          and the corresponding declaration lines.
c   it is now set for a microcomputer that has 64KB segment limit.
c
      na=1153
      nb=8000
      la=1436
c
c files:
c   1=monitoring (console on PC, file on WS),
c   3=data file for the continued case, 5=input, 6=output, 7=graphics
c
      open (1,file='pframe.mon')
c      open (1,file='con')
c         # 3 will be opened after 'icont' is input
      open (5,file='pframe.dat')
c         # 6 will be opened below
      open (6,file='pframe.res')
      open (7,file='pframe.grp')
c
      write(1,850)
      write(6,850)
c
c start the case
c
c          Data (1) or (17), and (2)
c
   10 continue
      read(5,100) title
      if( title(1).eq.aend ) go to 60
      write(6,300) title
      write(1,800) title
      read(5,200) n,node,icont
      write(6,400) n,node
c
c set the data area for each case
c
      n0=3*node
      n01=n0+1
      n02=n01+1
c
      l1=1
      l2=l1+n
      l3=l2+n
      l4=l3+n02
      l5=l4+n01
      llast=l5+n01-1
      if( llast.gt.la ) go to 90
c
      call prep ( ia(l1),ia(l2),ia(l3),ia(l4),n,n01,n02,node,mlast )
c
      m1=1
      if( mlast.gt.nb ) go to 30
      n1=1
      n2=n1+n0
      n3=n2+n0
      nlast=n3+n01-1
      if( nlast.gt.na ) go to 40
c
c file #3 is opened here (new or old) for the continued case
c
      if( icont.ne.0 ) write(6,950)
      open (3,file='pframe.cnt',form='unformatted')
c
c clear the data space if necessary
c
      do 20 i=1,nlast
      a(i)=0.d0
   20 continue
c
c start the main routine
c
      call beam ( a(n1),a(n2),a(n3),ia(l1),ia(l2),ia(l3),ia(l4),
     *                               ia(l5),b,c,n,n0,n01,mlast,icont )
      go to 10
c
c data area size error
c
   40 continue
      write(6,500) nlast,na
      go to 70
c
   30 continue
      write(6,600) mlast,nb
      go to 70
c
   90 continue
      write(6,650) llast,la
      go to 70
c
c end of execution
c                            successfully
   60 continue
      write(6,700)
c                            with size error
   70 continue
c
c files are closed before stop
c
      close (7)
      close (6)
      close (5)
      close (3)
      close (1)
      stop
c
  100 format(20a4)
  200 format(6i5)
  300 format(///' ',20a4/)
  400 format(' ','No. of Timoshenko-beam elements =',i5,' :',10x,
     *           'No. of nodes =',i5/)
  500 format(/' ','A :',i10,' is greater than',i10,'.')
  600 format(/' ','B and C:',i10,' is greater than',i10,'.')
  650 format(/' ','IA:',i10,' is greater than',i10,'.')
  700 format(/' ','All done !!!!')
  800 format(' ',20a4/)
  850 format(' ','Elastic Analysis of Plane Timoshenko Frame',
     *           6x,': Version 09.03.91 of 08.19.88'/)
  950 format(' ','*****     This is a continued case.    *****')
      end
c
      subroutine prep ( nnlft,nnrgt,imx,msl,n,n01,n02,node,mlast )
c
c  *********************   P R E P A R A T I O N   *******************
c
c    Set the size of 1-D array for the global tangential
c               stiffness matrix stored by the method of skyline
c
      implicit real*8 (a-h,o-z)
      dimension nnlft(n),nnrgt(n),imx(n02),msl(n01)
c
c  Data (3)
c
      do 10 i=1,n
      read(5,100) iel,nnlft(iel),nnrgt(iel)
      if( i.eq.iel ) go to 10
      write(6,200)
      stop
   10 continue
c
c  Calculate column hight of the global tangential stiffness matrix
c
      do 20 i=1,node
      j=3*(i-1)+1
      imx(j)=0
   20 continue
      do 30 i=2,node
      ii=node+2-i
      l=3*(ii-1)+1
      do 30 j=1,n
      jj=nnlft(j)
      if( nnrgt(j).gt.nnlft(j) ) jj=nnrgt(j)
      if( ii.ne.jj ) go to 30
      k=iabs(nnlft(j)-nnrgt(j))
      if( imx(l).lt.k ) imx(l)=k
   30 continue
      do 40 i=1,node
      j=3*(i-1)+1
      imx(j)=3*imx(j)+1
      imx(j+1)=imx(j)+1
      imx(j+2)=imx(j)+2
   40 continue
      imx(n01)=n01
c
c  position of diagonal elements in the skyline storage
c
      k=imx(1)
      do 50 i=2,n02
      j=imx(i)
      imx(i)=imx(i-1)+k
      k=j
   50 continue
c
c  maximum size for the array of global stiffness
c
      mlast=imx(n02)-1
c
c  row # of the upper end of column
c
      do 60 i=1,n01
      msl(i)=1+i+imx(i)-imx(i+1)
   60 continue
c
      return
c
  100 format(3i5)
  200 format(' ','Data error !!!!!   in data (3)')
      end
c
      subroutine beam ( d,add,anegtm,nnlft,nnrgt,imx,msl,nnegtm,
     *                                stuh,stlh,n,n0,n01,mlast,icont )
c
c ****************   M A I N   R O U T I N E   ***********************
c
      implicit real*8 (a-h,o-z)
      character*10 aload(3)
      dimension d(n0),add(n0),stuh(mlast),stlh(mlast),anegtm(n01)
      dimension imx(n01),msl(n01),nnlft(n),nnrgt(n),nnegtm(n01)
      dimension stl(6,6),fl(6),t(3,3),tt(3,3),c(3,6),
     *          dlcl(6),flcl(6),tang(6,6)
      common / bound  / ddf(64),idf(64),idd(64)
      common / member / ang(64),bet(64),alph(64),
     *                  ratio(64),ratiob(64),ielang(64)
      common / hinge  / sthng(6,80),dhng(80),ddhng(80),
     *                  flhng(80),dtmp(6),ielhng(80)
      common / inpi   / ninc,nlinc,ndcon,nout,itmx,nhng
      data aload / 'Horizontal' , 'Vertical  ' , 'Moment    ' /
c
c *********************   P H A S E   I   ****************************
c
c                 preparation and initialization
c
      call initlz ( t,tt,tang,c,p,dp,adp,detold,idtold )
c
c                    input data for each case
c
      call input ( d,r,eps,nnlft,nnrgt,n,n0 )
c
c       input data from 'pframe.cnt' for the continued case
c
      idfset=0
      if( icont.eq.0 ) go to 20
      idfset=1
      read(3) dhng,d,p,add,adp,ddf
      rewind 3
   20 continue
c
c                 output of modified load ratio
c
      if( r.lt.0.d0.or.idfset.eq.0 ) go to 95
      write(6,300)
      do 90 i=1,nlinc
      jj=idf(i)-int((idf(i)-1)/3)*3
      ii=(idf(i)-jj)/3+1
      write(6,400) i,aload(jj),ddf(i),ii
   90 continue
   95 continue
c
c ********************   P H A S E   II   ****************************
c
c                          start solving
c
      do 10 inc=1,ninc
c
c ********************   P H A S E   II - 1   ************************
c
      rnew=dabs(r)
      iter=0
c
c              remember values at the previous step
c
      write(3) dhng,d,p,add,adp,ddf
      rewind 3
   30 continue
      iter=iter+1
      if( iter.le.itmx ) go to 40
c
c                       too many iterations;
c       start from the previous step with small arc length
c
      iter=0
      rnew=rnew/2.d0
      read(3) dhng,d,p,add,adp,ddf
      rewind 3
      go to 30
   40 continue
c
c   initialization of unbalanced force and tangential stiffness
c
      call setfst ( stuh,stlh,imx,add,p,adp,rnew,iter,n0,n01,mlast )
c
c *******************   P H A S E   II - 2   *************************
c
c        form the local tangent stiffness matrix 'stl' and
c               calculate element force vector 'fl'
c
      call ready ( d,stuh,stlh,imx,nnlft,nnrgt,stl,fl,t,tt,c,
     *                            dlcl,flcl,tang,iter,mlast,n01,n0,n )
c
c *******************   P H A S E   II - 3   *************************
c
c       take the essential boundary conditions into account
c
      call bdryc ( stuh,stlh,imx,msl,n01,mlast )
c
c *******************   P H A S E   II - 4   *************************
c
c                            solve it
c
      call solve ( stuh,stlh,dp,imx,msl,nnegtm,anegtm,
     *                                         det,idt,jdt,n01,mlast )
c
      if( nhng.eq.0 ) go to 50
c
c                           left-hinged
c
      call lhinge ( stlh,imx,nnlft,nnrgt,n,n01,mlast )
c
   50 continue
c
c *******************   P H A S E   II - 5   *************************
c
c      have obtained the answer to the simultaneous equation
c
      call answer ( stlh,add,d,imx,adp,dp,relerr,iter,n01,n0,mlast )
c
c      modify force vector ddf(i) for stability at the very first step
c
      if( r.lt.0.d0.or.idfset.eq.1 ) go to 60
      addmx=0.d0
      do 70 i=1,n0
      if( addmx.ge.dabs(add(i)) ) go to 70
      addmx=dabs(add(i))
   70 continue
      if( addmx.eq.0.d0 ) addmx=adp
      addmx=adp/addmx
      read(3) dhng,d,p,add,adp,ddf
      rewind 3
      do 80 i=1,nlinc
      ddf(i)=addmx*ddf(i)
   80 continue
      idfset=1
      go to 20
   60 continue
c
c                        check convergence
c
      write(1,200) iter,relerr,det,idt
      if( relerr.gt.eps ) go to 30
c
c                if it is OK, then output results
c
c *******************   P H A S E   II - 6   *************************
c
c             output both global and local quantities
c
      call final ( d,add,nnlft,nnrgt,t,dlcl,tang,p,adp,rold,detold,
     *                 idtold,det,idt,jdt,nnegtm,anegtm,inc,n,n0,n01 )
c
   10 continue
c
c *******************   P H A S E   III   ****************************
c
c                          one case done
c
      write(6,100)
c
c    save date in 'pframe.cnt' for the possible continued case
c
      write(3) dhng,d,p,add,adp,ddf
      rewind 3
c
      return
c
  100 format(' ','One case done !!!'//)
  200 format('+','iter=',i5,':  err=',1pd10.2,':  det=',0pf5.2,'D',i5)
  300 format(/' ','Applied load ratio is modified for stability')
  400 format(' ',10x,'#',i3,': ',a10,' load (its modified ratio = ',
     *                      1pd15.7,') is applied at node no.',i5,'.')
      end
c
      subroutine initlz ( t,tt,tang,c,p,dp,adp,detold,idtold )
c
c **************   I N I T I A L I Z A T I O N    ********************
c
      implicit real*8 (a-h,o-z)
      dimension t(3,3),tt(3,3),tang(6,6),c(3,6)
      common / hinge  / sthng(6,80),dhng(80),ddhng(80),
     *                  flhng(80),dtmp(6),ielhng(80)
c
      do 10 i=1,3
      do 10 j=1,3
      t(i,j)=0.d0
      tt(i,j)=0.d0
      tang(i,j)=0.d0
      tang(i+3,j)=0.d0
      tang(i,j+3)=0.d0
      tang(i+3,j+3)=0.d0
      c(i,j)=0.d0
      c(i,j+3)=0.d0
   10 continue
      c(1,1)=-1.d0
      c(2,2)=-1.d0
      c(3,3)=-1.d0
      c(1,4)=1.d0
      c(2,5)=1.d0
      c(3,6)=1.d0
      t(3,3)=1.d0
      tang(3,3)=1.d0
      tang(6,6)=1.d0
      do 20 i=1,80
      dhng(i)=0.d0
      ddhng(i)=0.d0
   20 continue
      p=0.d0
      dp=0.d0
      adp=1.d0
      detold=1.d0
      idtold=0
c
      return
      end
c
      subroutine input ( d,r,eps,nnlft,nnrgt,n,n0 )
c
c ****************   I N P U T   R O U T I N E   ********************
c
      implicit real*8 (a-h,o-z)
      character*10 aload(3),adisp(3)
      character*13 bload(6),bdisp(6)
      dimension d(n0),nnlft(n),nnrgt(n),
     *          sc(3,3),sd(3,3),scsc(3,3),sdsd(3,3)
      common / hinge  / sthng(6,80),dhng(80),ddhng(80),
     *                  flhng(80),dtmp(6),ielhng(80)
      common / bound  / ddf(64),idf(64),idd(64)
      common / member / ang(64),bet(64),alph(64),
     *                  ratio(64),ratiob(64),ielang(64)
      common / inpi   / ninc,nlinc,ndcon,nout,itmx,nhng
      common / graph  / xgrph(8),nlgrph,ndgrph,negrph(8),nfgrph(8)
      common / stiff  / se(3,3,64),sf(3,3,64),sese(3,3,64),sfsf(3,3,64)
      common / switch / ioptg,iext,model
      data aload / 'Horizontal' , 'Vertical  ' , 'Moment    ' /
      data bload / 'Axial (Left) ' , 'Shear (Left) ' , 'Moment(Left) ' ,
     *             'Axial (Right)' , 'Shear (Right)' , 'Moment(Right)' /
      data adisp / 'Horizontal' , 'Vertical  ' , 'Rotational' /
      data bdisp / 'Horiz.(Left) ' , 'Vert. (Left) ' , 'Rotatn(Left) ' ,
     *             'Horiz.(Right)' , 'Vert. (Right)' , 'Rotatn(Right)' /
c
c Data (4)
c
      pi=4.d0*datan(1.d0)
      read(5,100) ioptg,iext,model
      if( iext.ne.0 ) ioptg=1
      if( ioptg.ne.0 ) write(6,650)
      if( iext.ne.0 ) write(6,690)
      if( model.ne.0 ) then
          write(6,710)
      else
          write(6,700)
      end if
c
c Data (5)
c
      read(5,200) iang
      write(6,620) iang
      if( iang.le.64 ) go to 50
      write(6,610) iang
      stop
   50 continue
c
c Data (6)
c
      do 120 i=1,iang
      read(5,400) ielang(i),ang(i),ratio(i),bet(i),alph(i),ratiob(i)
      ang(i)=pi*ang(i)/1.8d2
      if( iext.ne.0 ) go to 120
c             form the local stiffness matrix when ax.ext. neglected
      call stiffm ( se(1,1,i),sf(1,1,i),sese(1,1,i),sfsf(1,1,i),
     *      sc,sd,scsc,sdsd,ratio(i),bet(i),alph(i),ratiob(i),1.d0 )
  120 continue
c
      jang=1
      write(6,660)
      do 40 i=1,n
      write(6,570) i,nnlft(i),nnrgt(i),ang(jang)*1.8d2/pi,
     *             ratio(jang),bet(jang),alph(jang),ratiob(jang)
      if( i.eq.ielang(jang) ) jang=jang+1
   40 continue
c
c Data (7)
c
      read(5,200) nhng
      if( nhng.le.80 ) go to 60
      write(6,670)
      stop
   60 continue
      if( nhng.eq.0 ) go to 130
c
c Data (8)
c
      write(6,680) nhng
      read(5,100) ( ielhng(i) , i=1,nhng )
      write(6,640) ( ielhng(i) , i=1,nhng )
  130 continue
c
c Data (9)
c
      read(5,200) iep,nout,itmx,r
      if( nout.eq.0 ) nout=1
      if( itmx.eq.0 ) itmx=10
      write(6,510) iep,nout,itmx,r
      eps=10.d0**(-iep)
c
c Data (10)
c
      read(5,200) ninc
      write(6,520) ninc
c
c Data (11)
c
      read(5,200) nlinc
      if( nlinc.eq.0 ) go to 20
      write(6,540) nlinc
c
c Data (12)
c
      do 30 i=1,nlinc
      read(5,300) ii,jj,ddf(i)
      idf(i)=3*(ii-1)+jj
      write(6,550) i,aload(jj),ddf(i),ii
   30 continue
   20 continue
      if( nlinc.le.64 ) go to 10
      write(6,530) nlinc
      stop
   10 continue
c
c Data (13)
c
      read(5,200) ndcon
      if( ndcon.le.64 ) go to 80
      write(6,590) ndcon
      stop
   80 continue
      if( ndcon.eq.0 ) go to 90
      write(6,600) ndcon
c
c Data (14)
c
      do 110 i=1,ndcon
      read(5,300) ii,jj,aa
      k=3*(ii-1)+jj
      d(k)=aa
      idd(i)=k
      write(6,580) adisp(jj),aa,ii
  110 continue
   90 continue
c
c Data (15) & (16)
c
      read(5,100) nlgrph,ndgrph
      if( nlgrph.lt.0.or.nlgrph.gt.nlinc ) nlgrph=0
      if( ndgrph.lt.0 ) nlgrph=0
      if( ndgrph.gt.8 ) ndgrph=8
      if( nlgrph.eq.0 ) then
          write(6,720)
      else
          write(6,730) nlgrph,ndgrph
          read(5,100) ( negrph(i),nfgrph(i), i=1,ndgrph )
          do 70 i=1,ndgrph
          if( nfgrph(i).gt.6 ) then
              write(6,740) bload(nfgrph(i)-6),negrph(i)
          else
              write(6,750) bdisp(nfgrph(i)),negrph(i)
          end if
   70     continue
      end if
c
      return
c
  100 format(16i5)
  200 format(3i5,f10.0)
  300 format(2i5,f10.0)
  400 format(i5,5f10.0)
  510 format(/' ','Factor of convergence (neg. power)  =',i5,',',17x,
     *            'Output step for elements            =',i5/
     *        ' ','Max. no. of iteration for one trial =',i5,',',17x,
     *            'Arc length for one increment        =',1pd15.7)
  520 format(/' ','Total no. of increments             =',i5)
  530 format(' ','NL:',i5,' is greater than 64.')
  540 format(/' ','No. of incremental load(s)          =',i5)
  550 format(' ',10x,'#',i3,': ',a10,' load (its ratio = ',1pd15.7,
     *                              ') is applied at node no.',i5,'.')
  570 format(' ',i5,3x,i5,3x,i5,4x,1pd15.7,2x,d15.7,3x,d15.7,4x,
     *                                                 d15.7,3x,d15.7)
  580 format(' ',10x,a10,' displacement,',1pd15.7,
     *                            ', is specified at node no.',i5,'.')
  590 format(' ','ND:',i5,' is greater than 64.')
  600 format(/' ','No. of constant displacement(s)     =',i5)
  610 format(' ','IANG:',i5,' is greater than 64.')
  620 format(/' ','No. of different member(s)      =',i5)
  640 format('+',20x,10i10/)
  650 format(' ','*****  Geometric stiffness is included *****')
  660 format(/' ','Element',2x,'Node #',2x,'Node #',3x,
     *            'Direction (deg)',2x,'Basic length/  ',2x,
     *            'Slenderness ratio',2x,'Young-s modulus/',4x,
     *            'EI/(Basic EI)'/' ',4x,'#',5x,'left',4x,'right',21x,
     *            'Element length',24x,'Shear modulus')
  670 format(' ','Data error !!!!!   in data (7)')
  680 format(/' ','No. of left-hinged element(s) =',i5/
     *        ' ','     Element no(s). :')
  690 format(' ','*****   Extension effect is included   *****')
  700 format(' ','<<<<<   BAR MODEL (Model B) is chosen  >>>>>')
  710 format(' ','<<<<< SPRING MODEL (Model A) is chosen >>>>>')
  720 format(/' ','No graphics data output'/)
  730 format(/' ','** Graphics data output ** y: load #',i5,' , x:',i5,
     *       ' components will be output (see list below)')
  740 format(' ',5x,a13,'  force   at     element #',i5)
  750 format(' ',5x,a13,' displacement at element #',i5)
      end
c
      subroutine stiffm ( sa,sb,sasa,sbsb,sc,sd,scsc,sdsd,ratio,
     *                                          bet,alph,ratiob,stq0 )
c
c ********   L O C A L   S T I F F N E S S   M A T R I C E S   *******
c
      implicit real*8 (a-h,o-z)
      dimension sa(3,3),sb(3,3),sasa(3,3),sbsb(3,3),
     *          sc(3,3),sd(3,3),scsc(3,3),sdsd(3,3)
      common / switch / ioptg,iext,model
c
      aa=ratio
      bb=bet*bet
      cc0=alph*aa*aa/bb/stq0/stq0
      cc=1.2d1*cc0
      cc3=cc*cc0
c
      if( model.eq.0 ) then
          cc1=1.d0+2.d1*cc0+1.d1*cc3
          cc2=2.d0*cc0+cc3
          cc4=0.d0
          cc6=1.d0
      else
          cc1=1.d0+2.d1*cc0
          cc2=2.d0*cc0-2.d0*cc3
          cc4=6.d0*cc3
          cc6=1.d0+6.d1*cc0
      end if
      cc5=cc2+cc4
c
      ff=1.d0+cc
      ffff=ff*ff
      fg=-2.d0*cc0/stq0
c local stiffness matrix in infinitesimal displacement theory
      do 50 i=1,3
      do 50 j=1,3
      sa(i,j)=0.d0
      sb(i,j)=0.d0
      sc(i,j)=0.d0
      sd(i,j)=0.d0
   50 continue
      sa(1,1)=-bb*aa
      sa(2,2)=-12.d0*aa*aa*aa/ff/stq0/stq0
      sa(2,3)=6.d0*aa*aa/ff/stq0
      sa(3,2)=-sa(2,3)
      sa(3,3)=(2.d0-cc)*aa/ff
      sb(1,1)=-sa(1,1)
      sb(2,2)=-sa(2,2)
      sb(2,3)=sa(3,2)
      sb(3,2)=sb(2,3)
      sb(3,3)=(4.d0+cc)*aa/ff
      if( iext.eq.0 ) go to 40
      sc(2,2)=(24.d0/ff+144.d0*fg*stq0/ffff)*aa*aa*aa/stq0/stq0/stq0
      sc(2,3)=(-6.d0/ff-72.d0*fg*stq0/ffff)*aa*aa/stq0/stq0
      sc(3,2)=-sc(2,3)
      sc(3,3)=-36.d0*fg*aa/ffff
      sd(2,2)=-sc(2,2)
      sd(2,3)=sc(3,2)
      sd(3,2)=sd(2,3)
      sd(3,3)=sc(3,3)
   40 continue
c local geometric stiffness matrix
      do 10 i=1,3
      do 10 j=1,3
      sasa(i,j)=0.d0
      sbsb(i,j)=0.d0
      scsc(i,j)=0.d0
      sdsd(i,j)=0.d0
   10 continue
      if( ioptg.eq.0 ) go to 20
      sasa(2,2)=-6.d0*aa/5.d0*cc1/ffff/stq0
      sasa(2,3)=(1.d-1-cc4)/ffff
      sasa(3,2)=-sasa(2,3)
      sasa(3,3)=stq0*(-1.d0/3.d1-cc5)/aa/ffff
      sbsb(2,2)=-sasa(2,2)
      sbsb(2,3)=sasa(3,2)
      sbsb(3,2)=sbsb(2,3)
      sbsb(3,3)=stq0*(2.d0/1.5d1+cc2)/aa/ffff
      if( iext.eq.0 ) go to 20
      scsc(2,2)=6.d0*aa*(cc1+4.d0*fg*stq0/ff)/5.d0/ffff/stq0/stq0
      scsc(2,3)=-12.d0*fg*cc6/5.d0/ffff/ff
      scsc(3,2)=-scsc(2,3)
      scsc(3,3)=(-1.d0/3.d1-cc5-6.d0*stq0*fg*cc6/5.d0/ff)/aa/ffff
      sdsd(2,2)=-scsc(2,2)
      sdsd(2,3)=scsc(3,2)
      sdsd(3,2)=sdsd(2,3)
      sdsd(3,3)=(2.d0/1.5d1+cc2-6.d0*stq0*fg*cc6/5.d0/ff)/aa/ffff
   20 continue
      do 30 j=1,3
      do 30 k=1,3
      sa(j,k)=ratiob*sa(j,k)
      sb(j,k)=ratiob*sb(j,k)
      sasa(j,k)=ratiob*sasa(j,k)
      sbsb(j,k)=ratiob*sbsb(j,k)
      if( iext.eq.0 ) go to 30
      sc(j,k)=ratiob*sc(j,k)
      sd(j,k)=ratiob*sd(j,k)
      scsc(j,k)=ratiob*scsc(j,k)
      sdsd(j,k)=ratiob*sdsd(j,k)
   30 continue
c
      return
      end
c
      subroutine setfst ( stuh,stlh,imx,add,p,adp,rnew,iter,
     *                                                  n0,n01,mlast )
c
c *********   S E T   F O R C E   A N D   S T I F F N E S S   ********
c
c   tangential stiffness matrix :
c       stuh : upper half triangle  with   diagonal elements
c                                 which are stored in stuh(imx(j)).
c       stlh : lower half triangle without diagonal elements
c
c   unbalanced force vector :
c       stlh(imx(j));i.e. supposed storage for diagonal part of stlh
c
      implicit real*8 (a-h,o-z)
      dimension stuh(mlast),stlh(mlast),imx(n01),add(n0)
      common / bound  / ddf(64),idf(64),idd(64)
      common / inpi   / ninc,nlinc,ndcon,nout,itmx,nhng
c
      do 30 i=1,mlast
      stuh(i)=0.d0
      stlh(i)=0.d0
   30 continue
      do 20 i=1,nlinc
      k=idf(i)
      j=imx(n01)+n01-k
      stuh(j)=-ddf(i)
      if( iter.eq.1 ) go to 20
      stlh(imx(k))=(p+adp)*ddf(i)
   20 continue
      if( iter.eq.1 ) stlh(imx(n01))=rnew
      do 10 i=1,n0
      j=imx(n01)+n01-i
      stlh(j)=add(i)
   10 continue
      stuh(imx(n01))=adp
c
      return
      end
c
      subroutine ready ( d,stuh,stlh,imx,nnlft,nnrgt,stl,fl,t,tt,c,
     *                            dlcl,flcl,tang,iter,mlast,n01,n0,n )
c
c ******   M A K E   F O R C E   A N D   S T I F F N E S S   *********
c
c              get 'stuh' and 'stlh' ready
c
      implicit real*8 (a-h,o-z)
      dimension d(n0),stuh(mlast),stlh(mlast)
      dimension imx(n01),nnlft(n),nnrgt(n)
      dimension stl(6,6),fl(6),t(3,3),tt(3,3),c(3,6),
     *          dlcl(6),flcl(6),tang(6,6)
      common / member / ang(64),bet(64),alph(64),
     *                  ratio(64),ratiob(64),ielang(64)
      common / hinge  / sthng(6,80),dhng(80),ddhng(80),
     *                  flhng(80),dtmp(6),ielhng(80)
      common / inpi   / ninc,nlinc,ndcon,nout,itmx,nhng
c
      iang=1
      kelhng=1
      lelhng=1
      call trns ( tang,ang(iang) )
c
      do 10 iel=1,n
c
      if( iel.le.ielang(iang) ) go to 40
      iang=iang+1
      call trns ( tang,ang(iang) )
   40 continue
c
      j1=3*(nnlft(iel)-1)
      j2=3*(nnrgt(iel)-1)
      do 50 i=1,3
      dtmp(i)=d(j1+i)
      dtmp(i+3)=d(j2+i)
   50 continue
      if( kelhng.gt.nhng ) go to 60
      if( iel.ne.ielhng(kelhng) ) go to 60
      dtmp(3)=dhng(kelhng)
      kelhng=kelhng+1
   60 continue
c
      call locald ( dtmp,dlcl,tang )
      call local ( dlcl,flcl,stl,t,tt,c,iang )
      call glbl ( fl,flcl,stl,tang )
c                                             left-hinged
      if( lelhng.gt.nhng ) go to 70
      if( iel.ne.ielhng(lelhng) ) go to 70
      flhng(lelhng)=fl(3)/stl(3,3)
      do 80 i=1,6
      sthng(i,lelhng)=stl(3,i)/stl(3,3)
      if( i.eq.3 ) go to 80
      fl(i)=fl(i)-stl(i,3)*flhng(lelhng)
      do 90 j=1,6
      if( j.eq.3 ) go to 90
      stl(i,j)=stl(i,j)-stl(i,3)*stl(3,j)/stl(3,3)
   90 continue
   80 continue
      fl(3)=0.d0
      sthng(3,lelhng)=0.d0
      do 110 i=1,6
      stl(i,3)=0.d0
      stl(3,i)=0.d0
  110 continue
      lelhng=lelhng+1
   70 continue
c
c form the global tangent stiffness matrix and
c calculate unbalanced force vector
c
      do 20 i=1,3
      j1=3*(nnlft(iel)-1)+i
      j2=3*(nnrgt(iel)-1)+i
      jhight=iabs(j1-j2)+i-3
c                                         unbalanced force
      if( iter.eq.1 ) go to 30
      stlh(imx(j1))=stlh(imx(j1))-fl(i)
      stlh(imx(j2))=stlh(imx(j2))-fl(i+3)
   30 continue
c                                         stiffness matrix
      k1=imx(j1)
      k2=imx(j2)
      stuh(k1)=stuh(k1)+stl(i,i)
      stuh(k2)=stuh(k2)+stl(i+3,i+3)
      if( i.eq.1 ) go to 120
      jed=i-1
c                                         diagonal part
      do 130 j=1,jed
      ij=i-j
      stuh(k1+j)=stuh(k1+j)+stl(ij,i)
      stuh(k2+j)=stuh(k2+j)+stl(ij+3,i+3)
      stlh(k1+j)=stlh(k1+j)+stl(i,ij)
      stlh(k2+j)=stlh(k2+j)+stl(i+3,ij+3)
  130 continue
  120 continue
c                                         off-diagonal part
      kbase=k2
      if( k1.lt.kbase ) go to 140
      kbase=k1
      do 150 j=1,3
      do 150 k=1,3
      stswap=stl(3+j,k)
      stl(3+j,k)=stl(j,3+k)
      stl(j,3+k)=stswap
  150 continue
  140 continue
      do 20 k=1,3
      ij=kbase+jhight+k-1
      stuh(ij)=stuh(ij)+stl(4-k,i+3)
      stlh(ij)=stlh(ij)+stl(i+3,4-k)
   20 continue
c
   10 continue
c
      return
      end
c
      subroutine trns ( t,ang )
c
c *******   C O O R D I N A T E   T R A N S F O R M A T I O N   ******
c
      implicit real*8 (a-h,o-z)
      dimension t(6,6)
c
      ca=dcos(ang)
      sa=dsin(ang)
      t(1,1)=ca
      t(2,2)=ca
      t(4,4)=ca
      t(5,5)=ca
      t(1,2)=-sa
      t(2,1)=sa
      t(4,5)=-sa
      t(5,4)=sa
c
      return
      end
c
      subroutine locald ( dglbl,dlcl,t )
c
c *************   L O C A L   D I S P L A C E M E N T   **************
c
c  get local displacement from global one
c
      implicit real*8 (a-h,o-z)
      dimension dglbl(6),dlcl(6),t(6,6)
c
      do 10 i=1,6
      dlcl(i)=0.d0
      do 10 j=1,6
      dlcl(i)=dlcl(i)+t(j,i)*dglbl(j)
   10 continue
c
      return
      end
c
      subroutine local ( d,fl,stl,t,tt,c,iang )
c
c ******************   L O C A L   Q U A N T I T I E S   *************
c
c form local tangent stiffness matrix 'stl' and force vector 'fl'
c
      implicit real*8 (a-h,o-z)
      dimension d(6),fl(6),stl(6,6),t(3,3),tt(3,3),c(3,6)
      dimension sl(6,3),dl(3),al(6,3),a(3,3),b(3,3),e(3,3),
     *          f(3,3),g(3,3),h(3,3),ql(6,3)
      common / member / ang(64),bet(64),alph(64),
     *                  ratio(64),ratiob(64),ielang(64)
      common / stiff  / se(3,3,64),sf(3,3,64),sese(3,3,64),sfsf(3,3,64)
      common / switch / ioptg,iext,model
      dimension sa(3,3),sb(3,3),sasa(3,3),sbsb(3,3),
     *          sc(3,3),sd(3,3),scsc(3,3),sdsd(3,3)
c
      aa=ratio(iang)
c
      call prelcl ( t,d,dl,stq,bet(iang),aa,cr,sr )
c
      if( iext.ne.0 ) then
          stq0=1.d0+stq/bet(iang)/bet(iang)
c                                  form the local stiffness matrix
          call stiffm ( sa,sb,sasa,sbsb,sc,sd,scsc,sdsd,ratio(iang),
     *                   bet(iang),alph(iang),ratiob(iang),stq0 )
      end if
c
      tt(1,1)=-sr
      tt(1,2)=-cr
      tt(2,1)=cr
      tt(2,2)=-sr
      c(1,3)=sr/aa
      c(2,3)=-cr/aa
      do 10 i=1,3
      do 10 j=1,3
      a(i,j)=0.d0
      b(i,j)=0.d0
      e(i,j)=0.d0
      f(i,j)=0.d0
      g(i,j)=0.d0
      h(i,j)=0.d0
      do 10 k=1,3
      if( iext.ne.0 ) then
          a(i,j)=a(i,j)+(sa(i,k)+stq*sasa(i,k))*t(j,k)
          b(i,j)=b(i,j)+(sb(i,k)+stq*sbsb(i,k))*t(j,k)
          e(i,j)=e(i,j)+(bet(iang)*bet(iang)*sasa(i,k)+
     *                                   sc(i,k)+stq*scsc(i,k))*t(j,k)
          f(i,j)=f(i,j)+(sa(i,k)+stq*sasa(i,k))*tt(j,k)
          g(i,j)=g(i,j)+(bet(iang)*bet(iang)*sbsb(i,k)+
     *                                   sd(i,k)+stq*sdsd(i,k))*t(j,k)
          h(i,j)=h(i,j)+(sb(i,k)+stq*sbsb(i,k))*tt(j,k)
      else
          a(i,j)=a(i,j)+(se(i,k,iang)+stq*sese(i,k,iang))*t(j,k)
          b(i,j)=b(i,j)+(sf(i,k,iang)+stq*sfsf(i,k,iang))*t(j,k)
          e(i,j)=e(i,j)+sese(i,k,iang)*t(j,k)
          f(i,j)=f(i,j)+(se(i,k,iang)+stq*sese(i,k,iang))*tt(j,k)
          g(i,j)=g(i,j)+sfsf(i,k,iang)*t(j,k)
          h(i,j)=h(i,j)+(sf(i,k,iang)+stq*sfsf(i,k,iang))*tt(j,k)
      end if
   10 continue
      do 20 i=1,3
      ii=3+i
      do 20 j=1,3
      sl(i,j)=0.d0
      sl(ii,j)=0.d0
      al(i,j)=0.d0
      al(ii,j)=0.d0
      ql(i,j)=0.d0
      ql(ii,j)=0.d0
      do 20 k=1,3
      sl(i,j)=sl(i,j)+t(i,k)*a(k,j)
      sl(ii,j)=sl(ii,j)+t(i,k)*b(k,j)
      al(i,j)=al(i,j)+tt(i,k)*a(k,j)+t(i,k)*f(k,j)
      al(ii,j)=al(ii,j)+tt(i,k)*b(k,j)+t(i,k)*h(k,j)
      ql(i,j)=ql(i,j)+t(i,k)*e(k,j)
      ql(ii,j)=ql(ii,j)+t(i,k)*g(k,j)
   20 continue
      do 30 i=1,6
      do 30 j=1,6
      stl(i,j)=0.d0
      do 30 k=1,3
      stl(i,j)=stl(i,j)+sl(i,k)*c(k,j)
   30 continue
      if( iext.ne.0 ) then
          bb=aa
      else
          bb=bet(iang)*bet(iang)*aa
      end if
      do 50 i=1,3
      ii=i+3
      a(i,1)=0.d0
      a(i,2)=0.d0
      b(i,1)=0.d0
      b(i,2)=0.d0
      do 50 j=1,3
      a(i,1)=a(i,1)+al(i,j)*dl(j)
      a(i,2)=a(i,2)+al(ii,j)*dl(j)
      b(i,1)=b(i,1)+bb*ql(i,j)*dl(j)
      b(i,2)=b(i,2)+bb*ql(ii,j)*dl(j)
   50 continue
      bb=-dl(1)*sr+dl(2)*cr
      do 40 i=1,6
      fl(i)=0.d0
      k=int((i-1)/3)
      l=i-3*k
      k=k+1
      stl(i,1)=stl(i,1)-cr*b(l,k)
      stl(i,2)=stl(i,2)-sr*b(l,k)
      stl(i,3)=stl(i,3)+a(l,k)+bb*b(l,k)
      stl(i,4)=stl(i,4)+cr*b(l,k)
      stl(i,5)=stl(i,5)+sr*b(l,k)
      do 40 j=1,3
      fl(i)=fl(i)+sl(i,j)*dl(j)
   40 continue
c
      return
      end
c
      subroutine prelcl ( t,d,dl,stq,bet,ratio,cr,sr )
c
c * P R E P A R A T I O N   F O R   L O C A L   Q U A N T I T I E S  *
c
      implicit real*8 (a-h,o-z)
      dimension t(3,3),d(6),dl(3)
c
      r=d(3)
      sr=dsin(r)
      cr=dcos(r)
      dl(1)=d(4)-d(1)-(cr-1.d0)/ratio
      dl(2)=d(5)-d(2)-sr/ratio
      dl(3)=d(6)-d(3)
      stq=bet*bet*ratio*(dl(1)*cr+dl(2)*sr)
      t(1,1)=cr
      t(1,2)=-sr
      t(2,1)=sr
      t(2,2)=cr
c
      return
      end
c
      subroutine glbl ( fg,fl,st,t )
c
c **************   G L O B A L   Q U A N T I T I E S   ***************
c
c  get global force and stiffness from local ones
c
      implicit real*8 (a-h,o-z)
      dimension fg(6),fl(6),st(6,6),t(6,6),a(6,6)
c
      do 10 i=1,6
      fg(i)=0.d0
      do 10 j=1,6
      fg(i)=fg(i)+t(i,j)*fl(j)
   10 continue
      do 20 i=1,6
      do 20 j=1,6
      a(i,j)=0.d0
      do 20 k=1,6
      a(i,j)=a(i,j)+st(i,k)*t(j,k)
   20 continue
      do 30 i=1,6
      do 30 j=1,6
      st(i,j)=0.d0
      do 30 k=1,6
      st(i,j)=st(i,j)+t(i,k)*a(k,j)
   30 continue
c
      return
      end
c
      subroutine bdryc ( stuh,stlh,imx,msl,n01,mlast )
c
c ***   E S S E N T I A L   B O U N D A R Y   C O N D I T I O N S   **
c
      implicit real*8 (a-h,o-z)
      dimension stuh(mlast),stlh(mlast)
      dimension imx(n01),msl(n01)
      common / bound  / ddf(64),idf(64),idd(64)
      common / inpi   / ninc,nlinc,ndcon,nout,itmx,nhng
c
      do 10 i=1,ndcon
      k=idd(i)
c                                  force and diagonal elements
      stlh(imx(k))=0.d0
      stuh(imx(k))=1.d0
      jst=imx(k)+1
      jed=imx(k+1)-1
c                                  off-diagonal elements
      if( jst.gt.jed ) go to 20
      do 30 j=jst,jed
      stuh(j)=0.d0
      stlh(j)=0.d0
   30 continue
   20 continue
      jst=k+1
      do 40 j=jst,n01
      if( msl(j).gt.k ) go to 40
      jk=imx(j)+j-k
      stuh(jk)=0.d0
      stlh(jk)=0.d0
   40 continue
   10 continue
c
      return
      end
c
      subroutine solve ( stuh,stlh,dp,imx,msl,nnegtm,anegtm,
     *                                         det,idt,jdt,n01,mlast )
c
c ****************   R O U T I N E   T O   S O L V E   ***************
c
c  Elimination method
c
      implicit real*8 (a-h,o-z)
      dimension stuh(mlast),stlh(mlast)
      dimension imx(n01),msl(n01),nnegtm(n01),anegtm(n01)
c
c  determinant = det D idt
c     jdt =  0 if all of determinants of partial matrix are positive
c         <> 0 otherwise
c
      det=stuh(1)
      idt=0
      jdt=0
      if( det.gt.0.d0 ) go to 80
      jdt=1
      nnegtm(jdt)=1
      anegtm(jdt)=stuh(1)
   80 continue
c
c  forward elimination
c
      do 10 j=2,n01
      ist=msl(j)+1
      ied=j-1
      if( ist.gt.ied ) go to 20
      do 30 i=ist,ied
      kst=msl(j)
      if( msl(i).gt.kst ) kst=msl(i)
      ked=i-1
      ij=imx(j)+j-i
      do 30 k=kst,ked
      kj=imx(j)+j-k
      ki=imx(i)+i-k
      stuh(ij)=stuh(ij)-stuh(kj)*stlh(ki)
      stlh(ij)=stlh(ij)-stlh(kj)*stuh(ki)
   30 continue
   20 continue
      do 40 i=msl(j),ied
      ij=imx(j)+j-i
      stuh(ij)=stuh(ij)/stuh(imx(i))
      stlh(ij)=stlh(ij)/stuh(imx(i))
   40 continue
      do 50 k=msl(j),ied
      kj=imx(j)+j-k
      stuh(imx(j))=stuh(imx(j))-stuh(kj)*stlh(kj)*stuh(imx(k))
      stlh(imx(j))=stlh(imx(j))-stlh(kj)*stlh(imx(k))
   50 continue
c
      if( j.eq.n01 ) go to 10
      if( stuh(imx(j)).gt.0.d0 ) go to 90
      jdt=jdt+1
      nnegtm(jdt)=j
      anegtm(jdt)=stuh(imx(j))
   90 continue
      det=det*stuh(imx(j))
      jjjdt=idint(dlog10(dabs(det)))
      idt=idt+jjjdt
      det=det/10.d0**(jjjdt)
c
   10 continue
c
c  backward substitution
c
      do 60 j=1,n01
      stlh(imx(j))=stlh(imx(j))/stuh(imx(j))
   60 continue
      do 70 j=2,n01
      i=n01+2-j
      ked=i-1
      do 70 k=msl(i),ked
      ii=imx(i)+i-k
      stlh(imx(k))=stlh(imx(k))-stuh(ii)*stlh(imx(i))
   70 continue
      dp=stlh(imx(n01))
c
      return
      end
c
      subroutine lhinge ( stlh,imx,nnlft,nnrgt,n,n01,mlast )
c
c ***********   L E F T   H I N G E D   E L E M E N T S   ************
c
      implicit real*8 (a-h,o-z)
      dimension stlh(mlast),imx(n01),nnlft(n),nnrgt(n)
      common / hinge  / sthng(6,80),dhng(80),ddhng(80),
     *                  flhng(80),dtmp(6),ielhng(80)
      common / inpi   / ninc,nlinc,ndcon,nout,itmx,nhng
c
      do 10 k=1,nhng
      kel=3*(nnlft(ielhng(k))-1)
      lel=3*(nnrgt(ielhng(k))-1)
      ddhng(k)=-flhng(k)
      do 20 i=1,3
      j=kel+i
      l=lel+i
      ddhng(k)=ddhng(k)-sthng(i,k)*stlh(imx(j))
     *                 -sthng(i+3,k)*stlh(imx(l))
   20 continue
      dhng(k)=dhng(k)+ddhng(k)
   10 continue
c
      return
      end
c
      subroutine answer ( stlh,add,d,imx,adp,dp,relerr,iter,
     *                                                  n01,n0,mlast )
c
c ***   G E T   A N S W E R S   F O R   D I S P L A C E M E N T S  ***
c
      implicit real*8 (a-h,o-z)
      dimension stlh(mlast),add(n0),d(n0),imx(n01)
      common / hinge  / sthng(6,80),dhng(80),ddhng(80),
     *                  flhng(80),dtmp(6),ielhng(80)
      common / inpi   / ninc,nlinc,ndcon,nout,itmx,nhng
      a=0.d0
      relerr=0.d0
      do 20 i=1,nhng
      a=a+dhng(i)*dhng(i)
      relerr=relerr+ddhng(i)*ddhng(i)
   20 continue
      do 10 i=1,n0
      if( iter.eq.1 ) add(i)=0.d0
      add(i)=add(i)+stlh(imx(i))
      d(i)=d(i)+stlh(imx(i))
      a=a+d(i)*d(i)
      relerr=relerr+stlh(imx(i))*stlh(imx(i))
   10 continue
      if( iter.eq.1 ) adp=0.d0
      adp=adp+dp
c                             Relative error of displacement vector
      relerr=dsqrt(relerr)/dsqrt(a)
c
      return
      end
c
      subroutine final ( d,add,nnlft,nnrgt,t,dlcl,tang,p,adp,rold,
     *          detold,idtold,det,idt,jdt,nnegtm,anegtm,inc,n,n0,n01 )
c
c ***   F I N A L   S T E P   F O R   O N E   I N C R E M E N T   ****
c
      implicit real*8 (a-h,o-z)
      dimension d(n0),add(n0),nnlft(n),nnrgt(n)
      dimension nnegtm(n01),anegtm(n01)
      dimension t(3,3),dlcl(6),tang(6,6)
      common / member / ang(64),bet(64),alph(64),
     *                  ratio(64),ratiob(64),ielang(64)
      common / hinge  / sthng(6,80),dhng(80),ddhng(80),
     *                  flhng(80),dtmp(6),ielhng(80)
      common / inpi   / ninc,nlinc,ndcon,nout,itmx,nhng
      common / graph  / xgrph(8),nlgrph,ndgrph,negrph(8),nfgrph(8)
      common / bound  / ddf(64),idf(64),idd(64)
c
      p=p+adp
      rold=adp*adp
      do 10 i=1,n0
      rold=rold+add(i)*add(i)
   10 continue
      rold=dsqrt(rold)
      do 20 i=1,n0
      add(i)=add(i)/rold
   20 continue
      adp=adp/rold
      detold=det
      idtold=idt
      if( inc.gt.1 ) go to 30
      write(6,300) p,det,idt,jdt
      go to 40
   30 continue
      write(6,200) p,det,idt,jdt
   40 continue
      write(1,100) inc,ninc,p,jdt
c
      iang=1
      kelhng=1
      call trns ( tang,ang(iang) )
      do 50 iel=1,n
      idiang=0
      if( iel.le.ielang(iang) ) go to 60
      iang=iang+1
      idiang=1
      call trns ( tang,ang(iang) )
   60 continue
      j1=3*(nnlft(iel)-1)
      j2=3*(nnrgt(iel)-1)
      do 70 i=1,3
      dtmp(i)=d(j1+i)
      dtmp(i+3)=d(j2+i)
   70 continue
      indhng=0
      if( kelhng.gt.nhng ) go to 80
      if( iel.ne.ielhng(kelhng) ) go to 80
      dtmp(3)=dhng(kelhng)
      kelhng=kelhng+1
      indhng=1
   80 continue
      call locald ( dtmp,dlcl,tang )
      call output ( dlcl,dtmp,t,iel,ielang(iang),
     *                    n,iang,nnlft(iel),nnrgt(iel),indhng,idiang )
c
   50 continue
c
      if( jdt.ne.0 )
     *         write(6,400) ( nnegtm(ijdt),anegtm(ijdt) , ijdt=1,jdt )
c
      if( nlgrph.le.0.or.nlgrph.gt.nlinc ) return
c                                                 graphics data output
      write(7,500) p*ddf(nlgrph),(xgrph(i),i=1,ndgrph)
c
      return
c
  100 format('+','Increment No. ',i5,' out of ',i5,' is finished.',
     *           '  P = ',1pd11.3,'   jdt = ',i5/)
  200 format(/' ','Elmt# Node',3x,'FaxialLL/EI',4x,'FltralLL/EI',6x,
     *            'MzL/EI',10x,'U/L',12x,'V/L',13x,'TH',4x,
     *            'P=',1pd11.3,' (D=',0pf5.2,'D',i4,',',i3,')')
  300 format(/' ',98x,'P=',1pd11.3,' (D=',0pf5.2,'D',i4,',',i3,')'/
     *        ' ','Elmt# Node',3x,'FaxialLL/EI',4x,'FltralLL/EI',6x,
     *            'MzL/EI',10x,'U/L',12x,'V/L',13x,'TH',10x,
     *            'Eps.Axial',6x,'Eps.Bend')
  400 format(' ','Negative Diagonal = ',4(i5,': ',1pd15.7,', '))
  500 format(' ',1p9e12.3)
      end
c
      subroutine output ( d,dglbl,t,iel,ieliel,
     *                              n,iang,nnlft,nnrgt,indhng,idiang )
c
c ***************   O U T P U T   R O U T I N E   ********************
c
      implicit real*8 (a-h,o-z)
      dimension dglbl(6),flgrph(6)
      dimension d(6),t(3,3),dl(3),a(3,3),b(3,3),sl(6,3),fl(6)
      dimension sa(3,3),sb(3,3),sasa(3,3),sbsb(3,3),
     *          sc(3,3),sd(3,3),scsc(3,3),sdsd(3,3)
      common / bound  / ddf(64),idf(64),idd(64)
      common / member / ang(64),bet(64),alph(64),
     *                  ratio(64),ratiob(64),ielang(64)
      common / graph  / xgrph(8),nlgrph,ndgrph,negrph(8),nfgrph(8)
      common / inpi   / ninc,nlinc,ndcon,nout,itmx,nhng
      common / stiff  / se(3,3,64),sf(3,3,64),sese(3,3,64),sfsf(3,3,64)
      common / switch / ioptg,iext,model
c
      indbc=0
      if( nlinc.eq.0 ) go to 40
      do 50 i=1,nlinc
      j=(idf(i)-1)/3+1
      if( j.ne.nnlft ) go to 50
      indbc=1
      go to 60
   50 continue
   40 continue
      do 70 i=1,ndcon
      j=(idd(i)-1)/3+1
      if( j.ne.nnlft ) go to 70
      indbc=1
      go to 60
   70 continue
   60 continue
      if( indbc.eq.1.or.iel.eq.1.or.iel.eq.n.or.
     *          iel.eq.ieliel.or.indhng.eq.1.or.idiang.eq.1 ) go to 90
c      if( indbc.eq.1.or.iel.eq.1.or.iel.eq.n ) go to 90
      i=iel
      j=int(i/nout)*nout
      if( i.ne.j ) go to 210
   90 continue
c
      call prelcl ( t,d,dl,stq,bet(iang),ratio(iang),cr,sr )
c
      if( iext.ne.0 ) then
          stq0=1.d0+stq/bet(iang)/bet(iang)
c                                  form the local stiffness matrix
          call stiffm ( sa,sb,sasa,sbsb,sc,sd,scsc,sdsd,ratio(iang),
     *                  bet(iang),alph(iang),ratiob(iang),stq0 )
      end if
c
      do 10 i=1,3
      do 10 j=1,3
      a(i,j)=0.d0
      b(i,j)=0.d0
      do 10 k=1,3
      if( iext.ne.0 ) then
          a(i,j)=a(i,j)+(sa(i,k)+stq*sasa(i,k))*t(j,k)
          b(i,j)=b(i,j)+(sb(i,k)+stq*sbsb(i,k))*t(j,k)
      else
          a(i,j)=a(i,j)+(se(i,k,iang)+stq*sese(i,k,iang))*t(j,k)
          b(i,j)=b(i,j)+(sf(i,k,iang)+stq*sfsf(i,k,iang))*t(j,k)
      endif
   10 continue
      do 20 i=1,3
      ii=i+3
      do 20 j=1,3
      sl(i,j)=0.d0
      sl(ii,j)=0.d0
      do 20 k=1,3
      sl(i,j)=sl(i,j)+t(i,k)*a(k,j)
      sl(ii,j)=sl(ii,j)+t(i,k)*b(k,j)
   20 continue
      do 30 i=1,6
      fl(i)=0.d0
      do 30 j=1,3
      fl(i)=fl(i)+sl(i,j)*dl(j)
   30 continue
      if( iel.ne.1.and.indbc.eq.0.and.
     *                          indhng.eq.0.and.idiang.eq.0 ) go to 80
      ebend=-fl(3)/bet(iang)/ratiob(iang)
c 2008/06/27 ---------VVVVVVVVVVVVVV
      flflax=-(fl(1)*dcos(d(3))+fl(2)*dsin(d(3)))
      flflsh=-(-fl(1)*dsin(d(3))+fl(2)*dcos(d(3)))
c      eext=-(fl(1)*dcos(d(3))+fl(2)*dsin(d(3)))/bet(iang)/bet(iang)
      eext=flflax/bet(iang)/bet(iang)
      eext=eext/ratiob(iang)
c      write(6,100) iel,nnlft,fl(1),fl(2),fl(3),
      write(6,100) iel,nnlft,flflax,flflsh,-fl(3),
     *                           dglbl(1),dglbl(2),dglbl(3),eext,ebend
   80 continue
      ebend=fl(6)/bet(iang)/ratiob(iang)
      flflax=fl(4)*dcos(d(6))+fl(5)*dsin(d(6))
      flflsh=-fl(4)*dsin(d(6))+fl(5)*dcos(d(6))
c      eext=(fl(4)*dcos(d(6))+fl(5)*dsin(d(6)))/bet(iang)/bet(iang)
      eext=flflax/bet(iang)/bet(iang)
      eext=eext/ratiob(iang)
c      write(6,100) iel,nnrgt,fl(4),fl(5),fl(6),
      write(6,100) iel,nnrgt,flflax,flflsh,fl(6),
     *                           dglbl(4),dglbl(5),dglbl(6),eext,ebend
c 2008/06/27 ---------^^^^^^^^^^^^^^
c
  210 continue
      do 200 i=1,ndgrph
      if( iel.ne.negrph(i) ) go to 200
c                                       graphics data saving
      if( nfgrph(i).lt.7 ) then
           xgrph(i)=dglbl(nfgrph(i))
      else
c 2008/06/27 ---------VVVVVVVVVVVVVV
           flgrph(1)=-(fl(1)*dcos(d(3))+fl(2)*dsin(d(3)))
           flgrph(2)=-(-fl(1)*dsin(d(3))+fl(2)*dcos(d(3)))
           flgrph(3)=-fl(3)
           flgrph(4)=fl(4)*dcos(d(6))+fl(5)*dsin(d(6))
           flgrph(5)=-fl(4)*dsin(d(6))+fl(5)*dcos(d(6))
           flgrph(6)=fl(6)
           xgrph(i)=flgrph(nfgrph(i)-6)
c 2008/06/27 ---------^^^^^^^^^^^^^^
      end if
  200 continue
      return
c
  100 format(' ',2i5,1p8d15.7)
      end

